/* **************************************************************************
// test_c_wrapper.c,v 1.10 2007/01/30 18:53:10 cnepveu Exp
//
// Copyright (c) 2007 Hexago Inc. All rights reserved.
// 
//   LICENSE NOTICE: You may use and modify this source code only if you
//   have executed a valid license agreement with Hexago Inc. granting
//   you the right to do so, the said license agreement governing such
//   use and modifications.   Copyright or other intellectual property
//   notices are not to be removed from the source code.
//
// Description:
//
// Author: Charles Nepveu
//
// Creation Date: November 2006
// __________________________________________________________________________
// *********************************************************************** */
#include <stdio.h>
#include <gw6cconfig/gw6c_c_wrapper.h>
#include <gw6cconfig/gw6cuistrings.h>
#include <stdlib.h>

#ifdef WIN32
#include <winsock2.h>
#else
#define WSACleanup()    (void)0
#endif


/* ----------------------------------------------------------------------- */
/* Windows socket initialization....                                       */
/* Because some validation routines perform socket operations.             */
void Init_Wsock( void )
{
#ifdef WIN32
  WSADATA wsaData;
  WORD wVersionRequested;
  wVersionRequested = MAKEWORD( 2, 2 );

  if( WSAStartup(wVersionRequested, &wsaData) != 0 )
  {
    printf( "Failed WSAStartup()\n" );;
    exit(1);
  }
#endif
}


/* ----------------------------------------------------------------------- */
int main( int argc, char* argv[] )
{
  if( argc != 2 )
  {
    printf("\nYou must specify a valid GW6C config file.\n");
    return 1;
  }


  char* buf = NULL;
  int i, iRet;
  short s;
  tBoolean b;

  Init_Wsock();
  if( (iRet = initialize( argv[1] )) == 0 )
  {
    printf("Validation successful!\n" );
  
    get_user_id( &buf );
    printf("%-26s -> %s\n", "User Id", buf );
    free(buf); buf = NULL;

    get_passwd( &buf );
    printf("%-26s -> %s\n", "Passwd", buf );
    free(buf); buf = NULL;

    get_server( &buf );
    printf("%-26s -> %s\n", "Server", buf );
    free(buf); buf = NULL;

    get_host_type( &buf );
    printf("%-26s -> %s\n", "Host Type", buf );
    free(buf); buf = NULL;

    get_prefixlen( &i );
    printf("%-26s -> %d\n", "Prefix len", i );

    get_ifprefix( &buf );
    printf("%-26s -> %s\n", "If Prefix", buf );
    free(buf); buf = NULL;

    get_dns_server( &buf );
    printf("%-26s -> %s\n", "DNS server", buf );
    free(buf); buf = NULL;

    get_gw6_dir( &buf );
    printf("%-26s -> %s\n", "Gateway6 Dir", buf );
    free(buf); buf = NULL;

    get_auth_method( &buf );
    printf("%-26s -> %s\n", "Authentication method", buf );
    free(buf); buf = NULL;

    get_auto_retry_connect( &b );
    printf("%-26s -> %s\n", "Auto Retry Connect", b == TRUE? "true" : "false" );

    get_retry_delay( &i );
    printf("%-26s -> %d\n", "Retry delay", i );

    get_keepalive( &b );
    printf("%-26s -> %s\n", "Keep-alive enabled", b == TRUE? "true" : "false" );

    get_keepalive_interval( &i );
    printf("%-26s -> %d\n", "KA Interval", i );

    get_tunnel_mode( &buf );
    printf("%-26s -> %s\n", "Tunnel mode", buf );
    free(buf); buf = NULL;

    get_if_tun_v6v4( &buf );
    printf("%-26s -> %s\n", "Tunnel iface V6V4", buf );
    free(buf); buf = NULL;

    get_if_tun_v6udpv4( &buf );
    printf("%-26s -> %s\n", "Tunnel iface V6UDPV4", buf );
    free(buf); buf = NULL;

    get_if_tun_v4v6( &buf );
    printf("%-26s -> %s\n", "Tunnel iface V4V6", buf );
    free(buf); buf = NULL;

    get_client_v4( &buf );
    printf("%-26s -> %s\n", "Client V4 address", buf );
    free(buf); buf = NULL;

    get_client_v6( &buf );
    printf("%-26s -> %s\n", "Client V6 address", buf );
    free(buf); buf = NULL;

    get_template( &buf );
    printf("%-26s -> %s\n", "Template", buf );
    free(buf); buf = NULL;

    get_proxy_client( &b );
    printf("%-26s -> %s\n", "Proxy client enabled", b == TRUE? "true" : "false" );

    get_broker_list_file( &buf );
    printf("%-26s -> %s\n", "Broker list file", buf );
    free(buf); buf = NULL;

    get_last_server_file( &buf );
    printf("%-26s -> %s\n", "last server used file", buf );
    free(buf); buf = NULL;

    get_always_use_last_server( &b );
    printf("%-26s -> %s\n", "Always use last server", b == TRUE ? "true" : "false" );

    get_log( "stderr", &s );
    printf("log_%-22s -> %d\n", "stderr", s );
    get_log( "console", &s );
    printf("log_%-22s -> %d\n", "console", s );
    get_log( "syslog", &s );
    printf("log_%-22s -> %d\n", "syslog", s );
    get_log( "file", &s );
    printf("log_%-22s -> %d\n", "file", s );

    get_log_filename( &buf );
    printf("%-26s -> %s\n", "Log filename", buf );
    free(buf); buf = NULL;

    get_log_rotation( &b );
    printf("%-26s -> %s\n", "Log rotation enabled", b == TRUE? "true" : "false" );

    get_log_rotation_sz( &i );
    printf("%-26s -> %d\n", "Log rot size(KB)", i );

    get_syslog_facility( &buf );
    printf("%-26s -> %s\n", "Syslog facility", buf );
    free(buf); buf = NULL;
  }
  else
  {
    if( iRet == -1 )
    {
      unsigned int* ppErr = NULL;
      int    iCount;
      int    i;
    
      printf("Failed to load the file %s:\n", argv[1]);
      get_config_errors( &iCount, &ppErr );
      
      for( i=0; i<iCount; i++ )
      {
        printf("  %s\n", get_ui_string(ppErr[i]) );
      }
      free( ppErr );
    }
    else
      printf("initialize() error:  %s\n", get_ui_string(iRet) );
  }

  un_initialize();
  WSACleanup();


  return 0;
}

