// **************************************************************************
// test_namevalue.cc,v 1.11 2007/01/30 18:53:10 cnepveu Exp
//
// Copyright (c) 2007 Hexago Inc. All rights reserved.
// 
//   LICENSE NOTICE: You may use and modify this source code only if you
//   have executed a valid license agreement with Hexago Inc. granting
//   you the right to do so, the said license agreement governing such
//   use and modifications.   Copyright or other intellectual property
//   notices are not to be removed from the source code.
//
// Description:
//
// Author: Charles Nepveu
//
// Creation Date: November 2006
// __________________________________________________________________________
// **************************************************************************
#include <gw6cconfig/gw6cconfig.h>
#include <gw6cconfig/gw6cuistrings.h>
#include <gw6cconfig/gw6cvalidation.h>
using namespace gw6cconfig;

#include <iostream>
#include <iomanip>

#ifdef WIN32
#include <winsock2.h>
#else
#define WSACleanup()    (void)0
#endif


#define TRY_OR_DO(X,Y)  try{ X; } catch(...){ Y; }


// Windows socket initialization....
// Because some validation routines perform socket operations.
void Init_Wsock( void )
{
#ifdef WIN32
  WSADATA wsaData;
  WORD wVersionRequested;
  wVersionRequested = MAKEWORD( 2, 2 );

  if( WSAStartup(wVersionRequested, &wsaData) != 0 )
  {
    cout << "Failed WSAStartup()" << endl;
    exit(1);
  }
#endif
}


void PrintConfig( const GW6CConfig* pConfig )
{
  string sValue;

  pConfig->Get_UserID( sValue );
  cout << setw(32) << setiosflags(ios::left) << "User ID: " << sValue << endl;

  pConfig->Get_Passwd( sValue );
  cout << setw(32) << setiosflags(ios::left) << "Password: " << sValue << endl;

  pConfig->Get_Server( sValue );
  cout << setw(32) << setiosflags(ios::left) << "Server: " << sValue << endl;

  pConfig->Get_HostType( sValue );
  cout << setw(32) << setiosflags(ios::left) << "Host type: " << sValue << endl;

  pConfig->Get_PrefixLen( sValue );
  cout << setw(32) << setiosflags(ios::left) << "Prefix Len: " << sValue << endl;

  pConfig->Get_IfPrefix( sValue );
  cout << setw(32) << setiosflags(ios::left) << "If Prefix: " << sValue << endl;

  pConfig->Get_DnsServer( sValue );
  cout << setw(32) << setiosflags(ios::left) << "DNS Server: " << sValue << endl;

  pConfig->Get_Gw6cDir( sValue );
  cout << setw(32) << setiosflags(ios::left) << "GW6C Dir: " << sValue << endl;

  pConfig->Get_AuthMethod( sValue );
  cout << setw(32) << setiosflags(ios::left) << "Auth. Method: " << sValue << endl;

  pConfig->Get_AutoRetryConnect( sValue );
  cout << setw(32) << setiosflags(ios::left) << "Auto Retry Connect: " << sValue << endl;

  pConfig->Get_RetryDelay( sValue );
  cout << setw(32) << setiosflags(ios::left) << "Retry Delay: " << sValue << endl;

  pConfig->Get_KeepAlive( sValue );
  cout << setw(32) << setiosflags(ios::left) << "Keep-alive enabled: " << sValue << endl;

  pConfig->Get_KeepAliveInterval( sValue );
  cout << setw(32) << setiosflags(ios::left) << "Keep-alive interval: " << sValue << endl;

  pConfig->Get_TunnelMode( sValue );
  cout << setw(32) << setiosflags(ios::left) << "Tunnel Mode: " << sValue << endl;

  pConfig->Get_IfTunV6V4( sValue );
  cout << setw(32) << setiosflags(ios::left) << "If Tunnel V6V4: " << sValue << endl;

  pConfig->Get_IfTunV6UDPV4( sValue );
  cout << setw(32) << setiosflags(ios::left) << "If Tunnel V6UDPV4: " << sValue << endl;

  pConfig->Get_IfTunV4V6( sValue );
  cout << setw(32) << setiosflags(ios::left) << "If Tunnel V4V6: " << sValue << endl;

  pConfig->Get_ClientV4( sValue );
  cout << setw(32) << setiosflags(ios::left) << "Client V4 addr.: " << sValue << endl;

  pConfig->Get_ClientV6( sValue );
  cout << setw(32) << setiosflags(ios::left) << "Client V6 addr.: " << sValue << endl;

  pConfig->Get_Template( sValue );
  cout << setw(32) << setiosflags(ios::left) << "Template: " << sValue << endl;

  pConfig->Get_ProxyClient( sValue );
  cout << setw(32) << setiosflags(ios::left) << "Proxy Client enabled: " << sValue << endl;

  pConfig->Get_BrokerLstFile( sValue );
  cout << setw(32) << setiosflags(ios::left) << "Broker List: " << sValue << endl;

  pConfig->Get_LastServFile( sValue );
  cout << setw(32) << setiosflags(ios::left) << "Last Server: " << sValue << endl;

  pConfig->Get_AlwaysUseLastSrv( sValue );
  cout << setw(32) << setiosflags(ios::left) << "Always Use Last Server: " << sValue << endl;

  pConfig->Get_Log( STR_LOGDEV_CONSOLE, sValue );
  cout << setw(32) << setiosflags(ios::left) << "log_"STR_LOGDEV_CONSOLE << sValue << endl;
  pConfig->Get_Log( STR_LOGDEV_STDERR, sValue );
  cout << setw(32) << setiosflags(ios::left) << "log_"STR_LOGDEV_STDERR << sValue << endl;
  pConfig->Get_Log( STR_LOGDEV_SYSLOG, sValue );
  cout << setw(32) << setiosflags(ios::left) << "log_"STR_LOGDEV_SYSLOG << sValue << endl;
  pConfig->Get_Log( STR_LOGDEV_FILE, sValue );
  cout << setw(32) << setiosflags(ios::left) << "log_"STR_LOGDEV_FILE << sValue << endl;

  pConfig->Get_LogFileName( sValue );
  cout << setw(32) << setiosflags(ios::left) << "Log Filename: " << sValue << endl;

  pConfig->Get_LogRotation( sValue );
  cout << setw(32) << setiosflags(ios::left) << "Log Rotation Enabled: " << sValue << endl;

  pConfig->Get_LogRotationSz( sValue );
  cout << setw(32) << setiosflags(ios::left) << "Log Rotation Size: " << sValue << endl;

  pConfig->Get_SysLogFacility( sValue );
  cout << setw(32) << setiosflags(ios::left) << "Sys Log Facility: " << sValue << endl;
}

void ModifConfig( GW6CConfig* pConfig )
{
  pConfig->Set_UserID( "Johnny" );
  TRY_OR_DO( pConfig->Set_Passwd( "Le mot de passe  johnny qui va peut-tre dpasser les 64 c"\
                                  "aractres..." ), pConfig->Set_Passwd( "Un mot de passe hyper secret") );
  pConfig->Set_Server( "i.wanna.have.legitimate.mittens.to.bang.on.the.counter.com" );
  TRY_OR_DO( pConfig->Set_HostType( "hfsfg" ), pConfig->Set_HostType( "router" ) );
  TRY_OR_DO( pConfig->Set_PrefixLen( "132" ), pConfig->Set_PrefixLen( "64" ) );
  pConfig->Set_IfPrefix( "eth0" );
  TRY_OR_DO( pConfig->Set_DnsServer( "a.dns.server.com" ), pConfig->Set_DnsServer( "server.com" ) );
#ifdef WIN32
  pConfig->Set_Gw6cDir( "c:\\temp" );
#else
  pConfig->Set_Gw6cDir( "/tmp" );
#endif
  TRY_OR_DO( pConfig->Set_AuthMethod( "bonjour" ), pConfig->Set_AuthMethod( "plain" ) );
  TRY_OR_DO( pConfig->Set_AutoRetryConnect( "maybe" ), pConfig->Set_AutoRetryConnect( "no" ) );
  pConfig->Set_RetryDelay( "0" );
  TRY_OR_DO( pConfig->Set_KeepAlive( "maybe" ), pConfig->Set_KeepAlive( "yes" ) );
  TRY_OR_DO( pConfig->Set_KeepAliveInterval( "184424790256706744073709551615" ), pConfig->Set_KeepAliveInterval( "16" ) );
  TRY_OR_DO( pConfig->Set_TunnelMode( "mode alpha" ), pConfig->Set_TunnelMode( "v6anyv4" ) );
  pConfig->Set_IfTunV6V4( "pet" );
  pConfig->Set_IfTunV6UDPV4( "et" );
  pConfig->Set_IfTunV4V6( "patapouf" );
  TRY_OR_DO( pConfig->Set_ClientV4( "255.255.255.224ceci ne devrait pas paraitre" ), pConfig->Set_ClientV4( "55.55.25.22" ) );
  pConfig->Set_ClientV6( "Fe80:BABA::2c67" );
  TRY_OR_DO( pConfig->Set_Template( "vax/vms" ), pConfig->Set_Template( "netbsd" ) );
  pConfig->Set_ProxyClient( "yes" );
  pConfig->Set_BrokerLstFile( "broker.hereitis.txt" );
  pConfig->Set_LastServFile( "last-patente.txt" );
  TRY_OR_DO( pConfig->Set_AlwaysUseLastSrv( "mayhaps" ), pConfig->Set_AlwaysUseLastSrv( "no" ) );
  TRY_OR_DO( pConfig->Set_Log( "ghandi", "b" ), TRY_OR_DO( pConfig->Set_Log( "stderr", "9" ), pConfig->Set_Log( "stderr", "3" ) ) );
  TRY_OR_DO( pConfig->Set_Log( "", "sfdg" ), TRY_OR_DO( pConfig->Set_Log( "", "" ), pConfig->Set_Log( "file", "3" ) ) );
  TRY_OR_DO( pConfig->Set_Log( "console", "-1" ), TRY_OR_DO( pConfig->Set_Log( "console", "4" ), pConfig->Set_Log( "console", "3" ) ) );
  TRY_OR_DO( pConfig->Set_Log( "syslog", "90f7d6g987698" ), TRY_OR_DO( pConfig->Set_Log( "syslog ", " " ), pConfig->Set_Log( "syslog", "3" ) ) );
  pConfig->Set_LogFileName( "hahahaha.txt" );
  pConfig->Set_LogRotation( "yes" );
  TRY_OR_DO( pConfig->Set_LogRotationSz( "1280" ), pConfig->Set_LogRotationSz( "128" ) );
  TRY_OR_DO( pConfig->Set_SysLogFacility( "tente  oxygne" ), pConfig->Set_SysLogFacility( "LOCAL4" ) );
}


void GW6CConfigTest( const string& aFileName )
{
  try
  {
    GW6CConfig* pConfig = new GW6CConfig();

    pConfig->Initialize( aFileName, AM_RW );
    if( pConfig->Load() )
    {
      cout << "**********************************************************************" << endl;
      PrintConfig( pConfig );
      
      cout << "**********************************************************************" << endl;
      ModifConfig( pConfig );
      PrintConfig( pConfig );
      
      cout << "**********************************************************************" << endl;
      pConfig->CancelChanges();
      PrintConfig( pConfig );
     
      cout << "**********************************************************************" << endl;
      ModifConfig( pConfig );
      pConfig->Save();
      pConfig->CancelChanges();
      PrintConfig( pConfig );
    }
    else
    {
      const vector<error_t>& vErrors = pConfig->GetValidationErrors();
      vector<error_t>::const_iterator iter;

      cout << "Configuration file is bad." << endl << endl;
      cout << "Errors reported: " << endl;
      
      for( iter=vErrors.begin(); iter!=vErrors.end(); iter++ )
        cout << "  " << get_ui_string(*iter) << endl;
    }

    delete pConfig;
  }
  catch( error_t nErr )
  {
    cout << "Unhandled exception: " << get_ui_string(nErr) << endl << endl;
  }
}


void PrintConfig( const NameValueConfig* pConfig )
{
  list<string>::const_iterator iter;
  list<string> lstNames;
  string sValue;

  pConfig->GetVariableNameList( lstNames );
  for( iter=lstNames.begin(); iter!=lstNames.end(); iter++ )
  {
    pConfig->GetVariableValue( *iter, sValue );
    cout << setw(32) << setiosflags(ios::left) << *iter << "= " << sValue << endl;  
  }
}

void ModifConfig( NameValueConfig* pConfig )
{
  pConfig->RemoveVariable("userid");
  // passwd
  // server
  pConfig->RemoveVariable("host_type");
  pConfig->RemoveVariable("prefixlen");
  pConfig->RemoveVariable("if_prefix");
  pConfig->RemoveVariable("dns_server");
  pConfig->RemoveVariable("gw6_dir");
  // auth_method
  pConfig->RemoveVariable("auto_retry_connect");
  pConfig->RemoveVariable("retry_delay");
  pConfig->RemoveVariable("keepalive");
  // keepalive_interval
  // tunnel_mode
  pConfig->RemoveVariable("if_tunnel_v6v4");
  pConfig->RemoveVariable("if_tunnel_v6udpv4");
  pConfig->RemoveVariable("if_tunnel_v4v6");
  pConfig->RemoveVariable("client_v4");
  // client_v6
  pConfig->RemoveVariable("template");
  pConfig->RemoveVariable("proxy_client");
  pConfig->RemoveVariable("broker_list");
  // last_server
  pConfig->RemoveVariable("always_use_same_server"); 
  // log_stderr
  // log_syslog
  // log_console
  // log_file
  pConfig->RemoveVariable("log_filename");  
  pConfig->RemoveVariable("log_rotation");  
  pConfig->RemoveVariable("log_rotation_size");  
  // syslog_facility 
}

void NameValueConfigTest( const string& aFileName )
{
  try
  {
    NameValueConfig* pConfig = new NameValueConfig( aFileName, AM_RW );
    pConfig->LoadConfiguration();
    
    cout << "======================================================================" << endl;
    PrintConfig( pConfig );
    
    cout << "======================================================================" << endl;
    ModifConfig( pConfig );
    PrintConfig( pConfig );
    
    cout << "======================================================================" << endl;
    pConfig->CancelConfiguration();    
    PrintConfig( pConfig );
    
    cout << "======================================================================" << endl;
    ModifConfig( pConfig );
    pConfig->ApplyConfiguration();
    pConfig->CancelConfiguration();    
    PrintConfig( pConfig );
    
    delete pConfig;
  }
  catch( unsigned int nErr )
  {
    cout << "Unhandled exception: " << get_ui_string(nErr) << endl << endl;
  }
}

void TestLoadDefault( const string& aConfFile, const string& aDefaultFile )
{
  try
  {
    GW6CConfig* pConfig = new GW6CConfig();

    pConfig->Initialize( aConfFile, AM_RW, aDefaultFile );
    pConfig->Load();

    cout << "**********************************************************************" << endl;    
    ModifConfig( pConfig );
    pConfig->Save();
    PrintConfig( pConfig );
    
    cout << "**********************************************************************" << endl;    
    pConfig->LoadDefaults();
    PrintConfig( pConfig );

    delete pConfig;
  }
  catch( unsigned int nErr )
  {
    cout << "Unhandled exception: " << get_ui_string(nErr) << endl << endl;
  }
}


// --------------------------------------------------------------------------
/*                           PROGRAM MAIN LINE                             */
// --------------------------------------------------------------------------
int main( const int argc, const char* argv[] )
{
  if( argc == 3 )
  {
    TestLoadDefault( argv[1], argv[2] );
  }
  else if( argc == 2 )
  {
    Init_Wsock();
    GW6CConfigTest( argv[1] );
    NameValueConfigTest( argv[1] );
    WSACleanup();
  }
  else
  {
    cout << "Please provide a Gateway6 Configuration file as argument." << endl;
    return 1;
  }

  return 0;
}
