/*
---------------------------------------------------------------------------
 gw6c-genconf.c,v 1.8 2007/01/30 20:08:49 dgregoire Exp
---------------------------------------------------------------------------
This source code copyright (c) Hexago Inc. 2002-2007.

  LICENSE NOTICE: You may use and modify this source code only if you
  have executed a valid license agreement with Hexago Inc. granting
  you the right to do so, the said license agreement governing such
  use and modifications.   Copyright or other intellectual property
  notices are not to be removed from the source code.
---------------------------------------------------------------------------

	Created by Dominic Gregoire on 11/01/07.
	Copyright 2007 Hexago. All rights reserved. 

	Used to create a config file for gw6c on the openwrt platform.
	Uses nvram parameters:
	
	gw6c_tunnel
	gw6c_username
	gw6c_password
	gw6c_server

	Creates the config in /var/run/gw6c/gw6c.conf
	
	The init.d script starts the client with this
	config in the parameters.

*/

#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>

#include <bcmnvram.h>

#define NVRAM_STATUS "gw6c_tunnel"
#define NVRAM_STATUS_DISABLED "disabled"
#define NVRAM_USERNAME "gw6c_username"
#define NVRAM_PASSWORD "gw6c_password"
#define NVRAM_SERVER "gw6c_server"

#define CONFIG_FILE_PATH "/var/run/gw6c/gw6c.conf"
#define CONFIG_USERID "userid"
#define CONFIG_USERID_DEFAULT "anonymous"
#define CONFIG_PASSWORD "passwd"
#define CONFIG_PASSWORD_DEFAULT ""
#define CONFIG_SERVER "server"
#define CONFIG_SERVER_DEFAULT "broker.freenet6.net"
#define CONFIG_TRAILER \
	"host_type=router\n" \
	"prefixlen=48\n" \
	"if_prefix=br0\n" \
	"gw6_dir=/usr/share/gw6c\n" \
	"auth_method=any\n" \
	"retry_delay=30\n" \
	"keepalive=yes\n" \
	"keepalive_interval=30\n" \
	"tunnel_mode=v6anyv4\n" \
	"if_tunnel_v6v4=sit1\n" \
	"if_tunnel_v6udpv4=tun\n" \
	"if_tunnel_v4v6=sit0\n" \
	"client_v4=auto\n" \
	"client_v6=auto\n" \
	"template=openwrt\n" \
	"proxy_client=no\n" \
	"last_server=/var/run/gw6c/tsp-last-server.txt\n" \
	"log_console=0\n" \
	"log_stderr=1\n" \
	"log_file=0\n" \
	"log_syslog=0"

static 
int write_out(FILE *f, char *fmt, ...) {
	va_list ap;
	int written = 0;

	if (f == NULL)
		return 0;

	va_start(ap, fmt);
	written = vfprintf(f, fmt, ap);
	va_end(ap);
	
	return written;
}

int main(int argc, int **argv) {
	FILE *f_config;
	char *string;

	// If the client is disabled
	// unlink the config and
	// exit
	//
	string = nvram_get(NVRAM_STATUS);
	
	if (string == NULL) {
		nvram_set(NVRAM_STATUS, NVRAM_STATUS_DISABLED);
		nvram_set(NVRAM_SERVER, CONFIG_SERVER_DEFAULT);
		exit(1);
	}
	else if ( (string != NULL) && (strcmp(string, NVRAM_STATUS_DISABLED) == 0) ) {
		unlink(CONFIG_FILE_PATH);
		exit(1);
	}

	// Open the config file for
	// total overwrite and destruction
	//
	f_config = fopen(CONFIG_FILE_PATH, "w");
	
	if (f_config == NULL) {
		printf("%s: could not open config file %s\n", argv[0], CONFIG_FILE_PATH);
		exit(1);
	}
	
	// Get the username
	//
	//
	string = nvram_get(NVRAM_USERNAME);
	if (string == NULL) {
		string = CONFIG_USERID_DEFAULT;
		nvram_set(NVRAM_USERNAME, string);
	}
	
	// Write it out
	//
	write_out(f_config, "%s=%s\n", CONFIG_USERID, string);
	

	// password
	//
	string = nvram_get(NVRAM_PASSWORD);
	if (string == NULL) {
		string = CONFIG_PASSWORD_DEFAULT;
		nvram_set(NVRAM_PASSWORD, string);
	}
	
	// Write it out
	//
	write_out(f_config, "%s=%s\n", CONFIG_PASSWORD, string);
	
	// server
	//
	string = nvram_get(NVRAM_SERVER);
	if (string == NULL) {
		string = CONFIG_SERVER_DEFAULT;
		nvram_set(NVRAM_SERVER, string);
	}
	
	// Write it out
	//
	write_out(f_config, "%s=%s\n", CONFIG_SERVER, string);
	
	// Write the rest of
	// a standard config
	//
	write_out(f_config, "%s\n", CONFIG_TRAILER);
	
	// And quit
	//
	fclose(f_config);
	
	return 0;
}
