// **************************************************************************
// servermsgtranslator.cc,v 1.5 2007/01/30 18:53:31 cnepveu Exp
//
// Copyright (c) 2007 Hexago Inc. All rights reserved.
// 
//   LICENSE NOTICE: You may use and modify this source code only if you
//   have executed a valid license agreement with Hexago Inc. granting
//   you the right to do so, the said license agreement governing such
//   use and modifications.   Copyright or other intellectual property
//   notices are not to be removed from the source code.
//
// Description:
//   Implementation of the ServerMsgTranslator class.
//
// Author: Charles Nepveu
//
// Creation Date: November 2006
// __________________________________________________________________________
// **************************************************************************
#include <gw6cmessaging/servermsgtranslator.h>
#include <assert.h>


namespace gw6cmessaging
{
// --------------------------------------------------------------------------
// Function : ServerMsgTranslator constructor
//
// Description:
//   Will initialize a new ServerMsgTranslator object.
//
// Arguments: (none)
//
// Return values: (N/A)
//
// --------------------------------------------------------------------------
ServerMsgTranslator::ServerMsgTranslator( void ) :
  MessageProcessor()
{
  MessageProcessor::m_eProcessorState = STATE_ENABLED;
}


// --------------------------------------------------------------------------
// Function : ServerMsgTranslator destructor
//
// Description:
//   Will clean-up space allocated during object lifetime.
//
// Arguments: (none)
//
// Return values: (N/A)
//
// --------------------------------------------------------------------------
ServerMsgTranslator::~ServerMsgTranslator( void )
{
}


// --------------------------------------------------------------------------
// Function : ProcessMessage
//
// Description:
//   Will verify the message type and call the proper translator.
//
// Arguments:
//   pMsg: Message* [IN], The message to process.
//
// Return values:
//   GW6CM_UIS__NOERROR: Successful operation.
//   GW6CM_UIS_MESSAGENOTIMPL: Message not implemented.
//   GW6CM_UIS_MSGPROCDISABLED: Message processing is disabled.
//   <any other error message>
//
// --------------------------------------------------------------------------
error_t ServerMsgTranslator::ProcessMessage( Message* pMsg )
{
  error_t retCode;
  assert( pMsg );


  // Process messages only if the message processor is enabled.
  if( m_eProcessorState == STATE_ENABLED )
  {
    // -------------------------------------
    // Verify what kind of message this is.
    // -------------------------------------
    switch( pMsg->msg.header._msgid )
    {
    case MESSAGEID_REQUEST_STATUSINFO:
      retCode = TranslateStatusInfoReq( pMsg->msg._data, pMsg->msg.header._datalen );
      break;

    case MESSAGEID_REQUEST_TUNNELINFO:
      retCode = TranslateTunnelInfoReq( pMsg->msg._data, pMsg->msg.header._datalen );
      break;

    case MESSAGEID_REQUEST_BROKERLIST:
      retCode = TranslateBrokerListReq( pMsg->msg._data, pMsg->msg.header._datalen );
      break;

    default:
      retCode = GW6CM_UIS_MESSAGENOTIMPL; // Unknown / invalid message.
      break;
    }
  }
  else
    retCode = GW6CM_UIS_MSGPROCDISABLED;

  // Return completion status.
  return retCode;
}


// --------------------------------------------------------------------------
// Function : TranslateStatusInfoReq
//
// Description:
//   Will .
//
// Arguments:
//   pData: BYTE* [IN], The raw data.
//   nDataLen: WORD [IN], The length of the raw data.
//
// Return values:
//   GW6CM_UIS__NOERROR: Successful operation.
//   any other value on error.
//
// --------------------------------------------------------------------------
error_t ServerMsgTranslator::TranslateStatusInfoReq( BYTE* pData, const WORD nDataLen )
{
  // No translation is required for requests.
  // i.e.: GUI doesn't send information in requests.
  return Recv_StatusInfoRequest();
}


error_t ServerMsgTranslator::TranslateTunnelInfoReq( BYTE* pData, const WORD nDataLen )
{
  // No translation is required for requests.
  // i.e.: GUI doesn't send information in requests.
  return Recv_TunnelInfoRequest();
}


error_t ServerMsgTranslator::TranslateBrokerListReq( BYTE* pData, const WORD nDataLen )
{
  // No translation is required for requests.
  // i.e.: GUI doesn't send information in requests.
  return Recv_BrokerListRequest();
}

} // namespace
