// **************************************************************************
// test_gw6cemulator.c,v 1.4 2007/01/30 18:53:31 cnepveu Exp
//
// Copyright (c) 2007 Hexago Inc. All rights reserved.
// 
//   LICENSE NOTICE: You may use and modify this source code only if you
//   have executed a valid license agreement with Hexago Inc. granting
//   you the right to do so, the said license agreement governing such
//   use and modifications.   Copyright or other intellectual property
//   notices are not to be removed from the source code.
//
// Description:
//   Gateway6 Client emulator - Used to test the Windows GUI end.
//
// Author: Charles Nepveu
//
// Creation Date: November 2006
// __________________________________________________________________________
// **************************************************************************
#include <gw6cmessaging/gw6c_c_wrapper.h>
#include <gw6cmessaging/clientmsgdataretriever.h>


#ifdef WIN32
#include <windows.h>
#define SLEEP(X)  Sleep(X)
#else
#include <unistd.h>
#define SLEEP(X)  usleep((X) * 1000)
#endif

// rand()
#include <stdlib.h>
#include <stdio.h>
#include <assert.h>


// --------------------------------------------------------------------------
char* MakeRandomHostName( void )
{
  const char* names[] = { "hey.", "domain.", "nerdz.", "people.", "help.", 
                          "gateway6.", "broker.", "bug.", "weird.", "bone.", "army." };
  const char* suffixes[] = { "net", "com", "org", "gov" };

  char buffer[2048];
  int  z = rand() % 5 + 1;  // Random number of names.

  buffer[0] = 0;
  for( ; z>0; z-- )
    strcat( buffer, names[rand()%(sizeof(names)/sizeof(names[0]))] );
  strcat( buffer, suffixes[rand()%(sizeof(suffixes)/sizeof(suffixes[0]))] );

  char* szName = (char*)malloc(strlen(buffer)+1);
  strcpy( szName, buffer );
  return szName;
}


// --------------------------------------------------------------------------
char* MakeRndIP( short ver )
{
  assert( ver==4 || ver==6 );
  char buffer[64];

  if(ver==4)
    snprintf( buffer, 64, "%d.%d.%d.%d", rand()%256, rand()%256, rand()%256, rand()%256 );
  else
    snprintf( buffer, 64, "%X%X:%X%X:%X%X:%X%X:%X%X:%X%X:%X%X:%X%X", 
              rand()%256, rand()%256, rand()%256, rand()%256, rand()%256, rand()%256, 
              rand()%256, rand()%256, rand()%256, rand()%256, rand()%256, rand()%256,
              rand()%256, rand()%256, rand()%256, rand()%256 );

  char* szIP = (char*)malloc(strlen(buffer) + 1);
  strcpy( szIP, buffer );
  return szIP;
}


// --------------------------------------------------------------------------
// Create a random status information.
error_t RetrieveStatusInfo( Gw6cStatusInfo** ppStatusInfo )
{
  assert( *ppStatusInfo == NULL );
  static Gw6cCliStatus eStatus = GW6C_CLISTAT__DISCONNECTEDIDLE;
  static short nCounter = 0;


  // Try to make an intelligent status choice, based on the last status.
  switch( eStatus )
  {
  case GW6C_CLISTAT__DISCONNECTEDIDLE:
    if( nCounter > (rand() % 6) + 5) { eStatus = GW6C_CLISTAT__CONNECTING; nCounter = 0; }
    break;

  case GW6C_CLISTAT__CONNECTING:
    if( nCounter > (rand() % 5) + 2) { eStatus = ((rand()%10)>7)?GW6C_CLISTAT__DISCONNECTEDERROR:GW6C_CLISTAT__CONNECTED; nCounter = 0; }
    break;

  case GW6C_CLISTAT__CONNECTED:
    if( nCounter > (rand() % 5) + 5) { eStatus = ((rand()%15)>12)?GW6C_CLISTAT__DISCONNECTEDERROR:GW6C_CLISTAT__CONNECTED; nCounter = 0; }
    break;

  case GW6C_CLISTAT__DISCONNECTEDERROR:
    if( nCounter > (rand() % 6) + 1) { eStatus = GW6C_CLISTAT__CONNECTING; nCounter = 0; };
    break;

  case GW6C_CLISTAT__DISCONNECTEDNORETRY:
    eStatus = GW6C_CLISTAT__DISCONNECTEDERROR;
    break;
  }
  nCounter++;


  // Create struct
  *ppStatusInfo = (Gw6cStatusInfo* )malloc( sizeof(Gw6cStatusInfo) );
  (*ppStatusInfo)->eStatus  = eStatus;
  (*ppStatusInfo)->nStatus  = GW6CM_UIS__NOERROR;

  return GW6CM_UIS__NOERROR;
}


// --------------------------------------------------------------------------
// Create a random tunnel information.
error_t RetrieveTunnelInfo( Gw6cTunnelInfo** ppTunnelInfo )
{
  assert( *ppTunnelInfo == NULL );
  static Gw6cTunnelInfo* stcInfo = NULL;

  if( stcInfo == NULL )
  {
    stcInfo = (Gw6cTunnelInfo*) malloc( sizeof(Gw6cTunnelInfo) );

    stcInfo->szBrokerName             = MakeRandomHostName();
    stcInfo->eTunnelType              = (Gw6cTunnelType) (rand() % 4);
    stcInfo->szIPV4AddrLocalEndpoint  = MakeRndIP(4);
    stcInfo->szIPV6AddrLocalEndpoint  = MakeRndIP(6);
    stcInfo->szIPV4AddrRemoteEndpoint = MakeRndIP(4);
    stcInfo->szIPV6AddrRemoteEndpoint = MakeRndIP(6);
    stcInfo->szDelegatedPrefix        = MakeRndIP(6);
    stcInfo->szUserDomain             = MakeRandomHostName();
    stcInfo->tunnelUpTime             = time(NULL) - (rand() % 3000);
  }
  *ppTunnelInfo = stcInfo;

  return GW6CM_UIS__NOERROR;
}


// --------------------------------------------------------------------------
// Create a random broker list.
error_t RetrieveBrokerList( Gw6cBrokerList** ppBrokerList )
{
  assert( *ppBrokerList == NULL );
  static Gw6cBrokerList* stcInfo = NULL;

  if( stcInfo == NULL )
  {
    stcInfo = (Gw6cBrokerList*) malloc( sizeof(Gw6cBrokerList) );

    stcInfo->szAddress = MakeRandomHostName();
    stcInfo->nDistance = rand() % 30;
    stcInfo->next = NULL;
  }
  *ppBrokerList = stcInfo;

  return GW6CM_UIS__NOERROR;
}


// --------------------------------------------------------------------------
void  FreeStatusInfo( Gw6cStatusInfo** ppStatusInfo )
{
  if( *ppStatusInfo != NULL )
  {
    free( *ppStatusInfo );
    *ppStatusInfo = NULL;
  }
}


// --------------------------------------------------------------------------
void  FreeTunnelInfo( Gw6cTunnelInfo** ppTunnelInfo )
{/*
  if( *ppTunnelInfo != NULL )
  {
    free( (*ppTunnelInfo)->szBrokerName );
    free( (*ppTunnelInfo)->szIPV4AddrLocalEndpoint );
    free( (*ppTunnelInfo)->szIPV6AddrLocalEndpoint );
    free( (*ppTunnelInfo)->szIPV4AddrRemoteEndpoint );
    free( (*ppTunnelInfo)->szIPV6AddrRemoteEndpoint );
    free( (*ppTunnelInfo)->szDelegatedPrefix );
    free( (*ppTunnelInfo)->szUserDomain );

    free( *ppTunnelInfo );*/
    *ppTunnelInfo = NULL;
  //}
}


// --------------------------------------------------------------------------
void  FreeBrokerList( Gw6cBrokerList** ppBrokerList )
{/*
  if( *ppBrokerList != NULL )
  {
    free( (*ppBrokerList)->szBrokerName );
    free( *ppBrokerList );*/
    *ppBrokerList = NULL;
  //}
}


// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
void DoAnswerOnly( void )
{
  initialize_messaging();

  for( ; ; ) SLEEP(1000);

  uninitialize_messaging();           // never reached...
}


// --------------------------------------------------------------------------
void DoSendRandom( void )
{
  initialize_messaging();
  for( ; ; )
  {
    switch( rand() % 3 )
    {
    case 0: send_status_info();  break;
    case 1: send_tunnel_info();  break;
    case 2: send_broker_list();  break;
    }
    SLEEP( (rand() % 3000) + 1000);   // Sleep between 1 and 4 seconds.
  }

  uninitialize_messaging();           // never reached...
}


// --------------------------------------------------------------------------
void DoManualSendMode( void )
{
  char choice=0;

  initialize_messaging();
  for( ; choice != '4' ; )
  {
    do    // Ask user for input
    {
      printf( "    Please choose message to send:\n" );
      printf( "      (1) Send a random status info message.\n" );
      printf( "      (2) Send a random tunnel info message.\n" );
      printf( "      (3) Send a random broker list message.\n" );
      printf( "      (4) QUIT\n\n" );
      printf( "    Choice: " );
      choice = getchar();
    }
    while( choice != '1'  &&  choice != '2'  &&  choice != '3'  &&  choice != '4' );

    switch( choice )
    {
    case '1': send_status_info();  break;
    case '2': send_tunnel_info();  break;
    case '3': send_broker_list();  break;
    }
  }

  uninitialize_messaging();
}


// --------------------------------------------------------------------------
// MAIN PROGRAM ENTRY
// --------------------------------------------------------------------------
int main( int argc, char* argv[] )
{
  char choice;

  do
  {
    printf( "\nGateway6 Client Emulator. v1.0.0.0\n" );
    printf( "    Please choose operating mode:\n" );
    printf( "      1. Answer incoming requests only.\n" );
    printf( "      2. Send random messages.\n" );
    printf( "      3. Enter manual send mode.\n\n" );
    printf( "    Choice: " );
    choice = getchar();
  }
  while( choice != '1'  &&  choice != '2'  &&  choice != '3' );


  switch( choice )
  {
  case '1': DoAnswerOnly();       break;
  case '2': DoSendRandom();       break;
  case '3': DoManualSendMode();   break;
  }

  return 0;
}
