/*
---------------------------------------------------------------------------
 lib.c,v 1.20 2006/09/24 01:01:59 dgregoire Exp
---------------------------------------------------------------------------
Copyright (c) 2001-2006 Hexago Inc. All rights reserved.

  LICENSE NOTICE: You may use and modify this source code only if you
  have executed a valid license agreement with Hexago Inc. granting
  you the right to do so, the said license agreement governing such
  use and modifications.   Copyright or other intellectual property
  notices are not to be removed from the source code.
---------------------------------------------------------------------------
*/

#include <stdio.h>
#include <string.h>

#include "platform.h"
#include "hex_strings.h"

#include "lib.h"

// these must be kept in sync with errors.h

char *TspErrorCodesArray[] = { "NO_ERROR",
			       "NO_ERROR_SHOW_HELP",
			       "TSP_ERROR",
			       "SOCKET_ERROR",
			       "INTERFACE_SETUP_FAILED",
			       "KEEPALIVE_TIMEOUT",
				   "KEEPALIVE_ERROR",
			       "TUNNEL_ERROR",
			       "TSP_VERSION_ERROR",
			       "AUTHENTICATION_ERROR",
			       "LEASE_EXPIRED",
			       "SERVER_SIDE_ERROR",
			       "INVALID_ARGUMENTS",
			       "MEMORY_ERROR",
			       "INVALID_SERVER",
			       "INVALID_CONFIG_FILE",
			       "INVALID_CLIENT_IPV4",
			       "INVALID_CLIENT_IPV6",
                   "LOGGING_CONFIGURATION_ERROR",
                   "BROKER_REDIRECTION",
                   "BROKER_REDIRECTION_ERROR",
                   "SOCKET_ERROR_CANT_CONNECT",
				   "INITIALIZATION_ERROR",
			       NULL
};

/*
   Check if all characters in Value are within AllowedChars list.
*/

int IsAll(char *AllowedChars, char *Value)
{
	if(Value) {
		for(;*Value; Value++) {
			if(strchr(AllowedChars, *Value) == NULL)
				return 0;
		}
	} else {
		return 0;
	}
	return 1;
}

/*
   Check to see if there is a value in the char *
   If not, then the value was not supplied.
*/

int IsPresent(char *Value)
{
	if(Value)
		if(strlen(Value))
			return 1;
	return 0;
}

/* This next function is very dangerous.
   If you can call be certain the array
   finished by NULL or it will do bad things.
   */

int GetSizeOfNullTerminatedArray(char **a) {
	int i;
	for (i = 0;;i++) {
		if (a[i] == NULL)
			return i;
	}
	return -1;
}


char *tspGetErrorByCode(int code) {
	static char buf[1024];
	int i;
	
	i = GetSizeOfNullTerminatedArray(TspErrorCodesArray);
	if (code < i && code > -1)
		return TspErrorCodesArray[code];
	else
		snprintf(buf, sizeof(buf), HEX_STR_NOT_DEF_AS_CLIENT_ERROR, code);
	return buf;
}




