/*
---------------------------------------------------------------------------
 net.c,v 1.17 2006/09/22 23:06:43 cnepveu Exp
---------------------------------------------------------------------------
Copyright (c) 2001-2006 Hexago Inc. All rights reserved.

  LICENSE NOTICE: You may use and modify this source code only if you
  have executed a valid license agreement with Hexago Inc. granting
  you the right to do so, the said license agreement governing such
  use and modifications.   Copyright or other intellectual property
  notices are not to be removed from the source code.
---------------------------------------------------------------------------
*/

#include <stdlib.h>
#include <string.h>
#include <sys/types.h>

#define _USES_SYS_SOCKET_H_
#define _USES_NETINET_IN_H_
#define _USES_ARPA_INET_H_
#define _USES_NETDB_H_

#include "platform.h"

#include "tsp_net.h"
#include "net.h"
#include "log.h"
#include "hex_strings.h"

struct in_addr *NetText2Addr(char *Address)
{
  struct addrinfo hints;
  struct addrinfo *res=NULL, *result=NULL;
  char addr_cp[MAXSERVER];

  static struct in_addr saddr;
  char *addr;

  /* Prepare hints structure */
  memset(&hints, 0, sizeof(struct addrinfo));
  hints.ai_family = PF_UNSPEC;
  
  /* copy the string before using strtok */
  strcpy(addr_cp, Address);
  
  /* be sure it is not an in-brackets v6 address */
  if ((strchr(Address, '[') != NULL) ||
      (strchr(Address, ']') != NULL))
	  goto error_v4;

  /* be sure no more than on ':' is used to specify
     port number (not IPv6 address without brakets! */
  if ((addr = strchr(Address, ':')) != NULL) {
	  if (strchr(addr+1,':') != NULL)
		  goto error_v4;
  }
  
  /* Remove port number if any */
  addr = addr_cp;
  strtok(addr_cp, ":");	  

  if ((getaddrinfo(addr, NULL, &hints, &res)) == 0) {

	  for (result = res; result; result = result->ai_next) {
		  if (result->ai_family != AF_INET)
			  continue;
		  memcpy(&saddr,
			 &((struct sockaddr_in *)result->ai_addr)->sin_addr,
			 sizeof(struct in_addr));
		  freeaddrinfo(res);
		  return &saddr;
	  }
  
  }

 error_v4:  
  /* Cannot resolve */
  Display(LOG_LEVEL_3, ELInfo, "NetText2Addr",
	  HEX_STR_SERVER_NOT_IPV4);

  if (res != NULL)
	  freeaddrinfo(res);

  return NULL;
}

#ifdef V4V6_SUPPORT
/*
  Added for v4v6
 */
/* @bug: The following routine should not return a pointer to a
 * resource allocated here. We should have the caller pass a pointer
 * to an allocated buffer, and its length. */
struct in6_addr *NetText2Addr6(char *Address)
{
  struct addrinfo hints;
  struct addrinfo *res=NULL, *result=NULL;
  static struct in6_addr saddr;  /* @bug: static bufs are not thread-safe. */
  char *addr;
  char addr_cp[MAXSERVER];
  
  /* The input string may be a literal IPv6 address or a FQDN. This
     routine expects all IPv6 address strings to be enclosed inside '[
     ]' */
  memset(&hints, 0, sizeof(struct addrinfo));
  hints.ai_family = PF_UNSPEC;

  /* copy the string before using strtok */
  strcpy(addr_cp, Address);
  
  /* Extract v6 address from brackets if any */
  addr = strchr(addr_cp, '[');
  if (addr != NULL) {
	  addr = strtok(addr_cp, "]");
	  if (addr != NULL) {
	    hints.ai_flags = AI_NUMERICHOST; /* getaddrinfo should not
						try to resolve address
						string. */
	    addr++; /* move pointer to first char. of address */
	  }
  } else {
    /* if no '[' in input string, pass string as-is to getaddrinfo
       (assuming FQDN) */
    addr = addr_cp;
    /* remove port argument if any */
    strtok(addr_cp, ":");
    /* be sure we don't have other ':'
       (IPv6 address without brackets?) */
    if (strchr(addr, ':') != NULL)
	    goto error_v6;
  }

  if ((getaddrinfo(addr, NULL, &hints, &res)) == 0) {

	  for (result = res; result; result = result->ai_next) {
		  if (result->ai_family != AF_INET6)
			  continue;
		  memcpy(&saddr,
			 &((struct sockaddr_in6 *)result->ai_addr)->sin6_addr,
			 sizeof(struct in6_addr));
		  freeaddrinfo(res);
		  return &saddr;
	  }
  
  }
  
 error_v6:
  /* Cannot resolve */
  Display(LOG_LEVEL_3, ELInfo, "NetText2Addr6",
	  HEX_STR_SERVER_NOT_IPV6);

  if (res != NULL)
	  freeaddrinfo(res);
  
  return NULL;
  
}
#endif /* V4V6_SUPPORT */

int NetReadLine(unsigned char *in, size_t il, unsigned char *line, size_t ll)
{
	int i;

	if (in == NULL || line == NULL)
		return -1;
	
	for (i = 0; i < il; i++) {
		if (i == ll) {
			line[i] = 0;
			return i;		/* end of input buffer reached */
		}
		
		*(line + i) = *(in + i);
		if (ll == i || *line == '\n')
			break;
	}
	
	line[i] = 0;
	return i;
}
